//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// WIRELESS MOUSETRAP MONITORING SYSTEM by Alberto Ricci Bitti (c) 2003      www.riccibitti.com
//
//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// MODULE:   FLASH.C
// PURPOSE:  Provides an easy-to-use interface for erasing a data page, programming a byte or a word.
//           Hides low-level ROM-memory calls from the application
// REQUIRES: The flash area to be erased/written must be free from the main program.
//           Main application must be relocated accordingly changing the linker file.
//           (see "default.prm" file in "PRM" folder)
// NOTES:    For more information see AN2346, AN1831
//           The program provides a patch to overcome a bug present on early version of the silicon
//           IMPORTANT: The patch works for the erasure of the FFB0-FFBF range using tabular data:
//           the user MUST check that the table matches FLASH content for the user-monitor version
//           he is currently using
//
////////////////////////////////////////////////////////////////////////////////////////////////////////


#include "hardware.h"
#include "flash.h"



#define EraRnge 0x2806  /*ROM routine address for erasing a page of cells*/
#define PgrRnge 0x2809  /*ROM routine address for programming one or more flash cells*/

//variables at absolute addresses required by ROM routines
static unsigned char            CtrlByt @ 0x88 = 0;     // 0 = page (not mass) erase
static unsigned char            CPUSpd  @ 0x89 = 13;    // 3.2 Mhz / 0.25 = 13
static const unsigned char *    LADDR   @ 0x8A;         // last address to be programmed
static unsigned char            DATA    @ 0x8C;         // data buffer to be programmed

static void flash_patch( void );



///////////////////////////////////////////////////////////////////////////////
// flash_erase_page - erases the whole page pointed by flash_address         //
///////////////////////////////////////////////////////////////////////////////

void flash_erase_page( const unsigned char * flash_address )
{
    CtrlByt = 0;
    CPUSpd = 13;
    _asm ldhx flash_address;
    _asm JSR EraRnge;
    EnableInterrupts;

    flash_patch();  //apply the patch - comment out if you are using newer silicon
                    // or you are not erasing the page at EE00-EEFF
}



///////////////////////////////////////////////////////////////////////////////
// flash_program_byte - writes a byte at a specified location                //
// the location must be erased prior to write it                             //
// do not rewrite a location more than once                                  //
///////////////////////////////////////////////////////////////////////////////

void flash_program_byte( const unsigned char * flash_address, unsigned char data_byte )
{
    CtrlByt = 0;
    CPUSpd = 13;
    LADDR = flash_address;  //program 1 byte, so last address=start address
    DATA  = data_byte;
    _asm ldhx flash_address;
    _asm JSR  PgrRnge;
    EnableInterrupts;
}



///////////////////////////////////////////////////////////////////////////////
// flash_program_word - writes a 2-byte value at a specified location        //
// the locations must be erased prior to write it                            //
// do not rewrite a location more than once                                  //
///////////////////////////////////////////////////////////////////////////////

void flash_program_word( const unsigned int * flash_address, unsigned int data_word )
{
    flash_program_byte( (const unsigned char *) flash_address, data_word >> 8 );
    flash_program_byte( 1+(const unsigned char *) flash_address, data_word & 0xFF );
}



///////////////////////////////////////////////////////////////////////////////
// flash_patch - repairs FLASH contents in the range FFB0-FFBF if they get   //
// accidentally erased because of a flaw in earlier silicon                  //
//                                                                           //
// IMPORTANT: The patch recovers the erasure of the FFB0-FFBF range using    //
// tabular data the user MUST check that the table matches FLASH content     //
// for the user-monitor version he is currently using                        //
///////////////////////////////////////////////////////////////////////////////

void flash_patch( void )
{
    //table of the lost contents in the range FFB0-FFBF
    const unsigned char patch[16] = { 0xC6, 0XFF, 0xC0, 0xB7, 
                                      0x38, 0x11, 0x00, 0x14, 
                                      0x0B, 0xC6, 0xFD, 0xFE, 
                                      0x20, 0x04, 0xFE, 0xFF };
    unsigned char * addr;
    unsigned char i;

    for (i=0, addr = (unsigned char *) 0xFFB0; i< sizeof(patch); i++, addr++)
        if (*addr != patch[i])      
        {
            //the content differs, reprogram it
            flash_program_byte( addr, patch[i] );
        };
}

