//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// WIRELESS MOUSETRAP MONITORING SYSTEM by Alberto Ricci Bitti (c) 2003      www.riccibitti.com
//
//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// MODULE:   LCD.C
// PURPOSE:  Compact, portable alphanumeric LCD driver (4-bit mode)
//           -  prints strings with embedded printf-style escapes ( \n, \r, \f )
//           -  prints numbers (unsigned int)
//           -  prints single characters
//           -  supports cursor location
//           -  supports any display rows/coumns size
//           -  supports connection on arbitrary ports and unsorted output pins 
// REQUIRES: Macros to set/clear individual LCD pins must be defined externally.
//           Wait routines (see wait.h)
// NOTES:    None
//
////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "hardware.h"
#include "wait.h"
#include "lcd.h"


#define LCD_COLUMNS 16    //assume a 16 column display
#define LCD_ROWS    2     //assume a 2 rows display
#define LCD_SIZE    ( LCD_COLUMNS * LCD_ROWS )


#define WAIT_15_mS wait_mS( 15 )
#define WAIT_40_uS wait_x12_uS(4)


///////////////////////////////////////////////////////////////////////////////
// You should not need to touch anything below this line                     //
///////////////////////////////////////////////////////////////////////////////

static unsigned char row = 0;           // current cursor position, row
static unsigned char column = 0;        // current cursor position, column



// the command sequence required to setup the display is stored in flash
// add here any additional command (e.g. special characters bitmaps)

static const unsigned char cmd_setup[] = { 0x33, 0x32, 0x28, 0x08, 0x0C, 0x06, 0x02 };
    //                                       |     |     |     |     |     |     |
    // set interface mode to 4 bits----------+     |     |     |     |     |     |
    // (continue set interface mode)---------------+     |     |     |     |     |
    // function set--------------------------------------+     |     |     |     |
    // display off---------------------------------------------+     |     |     | 
    // display on----------------------------------------------------+     |     |
    // entry mode set------------------------------------------------------+     |
    // home----------------------------------------------------------------------+



///////////////////////////////////////////////////////////////////////////////
// LCD_nibble - puts a 4-bit command to  the display                         //
///////////////////////////////////////////////////////////////////////////////

static void LCD_nibble( unsigned char x )
{
    LCD_DB7 = get_bit(x, 3);    //extract bit 3 from byte x
    LCD_DB6 = get_bit(x, 2);    //...then bit 2, 1 and 0
    LCD_DB5 = get_bit(x, 1);    
    LCD_DB4 = get_bit(x, 0);

    LCD_E = 1;                  //strobe ENABLE line
    LCD_E = 0;
}


 
///////////////////////////////////////////////////////////////////////////////
// LCD_write - puts a byte to  the display as two 4-bit nibbles              //
// keeps current RS setting, the byte can be either a command or data        //
///////////////////////////////////////////////////////////////////////////////

static void LCD_write( unsigned char c )
{
    LCD_nibble( c >> 4 );
    LCD_nibble( c );
    WAIT_40_uS;
}



///////////////////////////////////////////////////////////////////////////////
// LCD_linefeed - movers cursor location to the start of the next line       //
// (rolls over display end)                                                  //
///////////////////////////////////////////////////////////////////////////////

static void LCD_linefeed(void)
{
    row = (unsigned char) ( ( row + 1 ) % LCD_ROWS );
    column = 0;
    LCD_locate( row, column );
}



///////////////////////////////////////////////////////////////////////////////
// LCD_locate - moves the cursor to a row, column                            //
///////////////////////////////////////////////////////////////////////////////

void LCD_locate( unsigned char r,unsigned char c )
{
    unsigned char ram_address = c + r*64;
    row = r;
    column = c;
    LCD_CD = 0;                         //switch to command mode
    LCD_write( 0x80 | ram_address );    //set display ram address
    LCD_CD = 1;                         //back to character mode
}



///////////////////////////////////////////////////////////////////////////////
// LCD_clear_to - advances the cursor from current location to row, column   //
// clearing all characterh in between                                        //
///////////////////////////////////////////////////////////////////////////////

void LCD_clear_to( unsigned char r, unsigned char c )
{
    while (r != row && c != column)
        LCD_putchar( ' ' );
}



///////////////////////////////////////////////////////////////////////////////
// LCD_putchar - writes a character and advances cursor location             //
// interprets printf-style escapes:                                          //
//     \r = "carriage return" brings cursor to leftmost column               //
//     \f = "form-feed" simulates a page clear                               //
//     \n = "newline" brings cursor to start of next line                    //
///////////////////////////////////////////////////////////////////////////////

void LCD_putchar( unsigned char c )
{
    switch( c )
    {
    case '\r':     //the "carriage return" escape brings cursor to leftmost column
            LCD_locate( row, 0);
            break;
            
    case '\f':     //the "form-feed" escape simulates a page clear
            {       unsigned char i;
                    LCD_locate( 0, 0 );
                    for ( i=0; i<LCD_SIZE; i++ )
                            LCD_write(' ');
            }
            break;
            
    case '\n':     //the "newline" escape brings to start of next line
            LCD_linefeed();
            break;          
            
    default:
            LCD_write( c );
            if ( ++column == LCD_COLUMNS )
                LCD_linefeed();
    }
}



///////////////////////////////////////////////////////////////////////////////
// LCD_initialize - initializes hardware and display                         //
///////////////////////////////////////////////////////////////////////////////

void LCD_initialize(void)
{
        unsigned char i;
        
        LCD_E_DDR   = 1;
        LCD_CD_DDR  = 1;
        LCD_DB7_DDR = 1;
        LCD_DB6_DDR = 1;
        LCD_DB5_DDR = 1;
        LCD_DB4_DDR = 1;

        LCD_CD = 0;
        LCD_E  = 0;
        WAIT_15_mS;
 
        for ( i=0; i < sizeof( cmd_setup ); i++)
        { 
                WAIT_15_mS;
                LCD_write( cmd_setup[i] ); 
        };

        WAIT_15_mS;

        LCD_CD = 1;
}
   
   
   
///////////////////////////////////////////////////////////////////////////////
// LCD_puts - puts a string to LCD                                           //      
// it accepts escape sequences embedded on string                            //
// e.g. "\fFIRST LINE\nSECOND LINE" clears the display, writes "FIRST LINE   //
// on the first line and "SECOND LINE" on the second                         //
///////////////////////////////////////////////////////////////////////////////

void LCD_puts( char * s )
{
    while (*s)
        LCD_putchar(*s++);
}


///////////////////////////////////////////////////////////////////////////////
// LCD_putn - prints a unsigned integer number to LCD                        //
///////////////////////////////////////////////////////////////////////////////

void LCD_putn( unsigned int n )
{
    unsigned char i = 0;
    unsigned char buf[5];       //integers need at most 5 digits    
    
    do                          //fill the buffer iteratively...
    {
        buf[i++] = '0' + n % 10;
        n = n / 10;
    } while (n != 0);
    
    
    while( i > 0)               //...and print it backwards
    {
        LCD_putchar( buf[ --i ] );
    }
}

