//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// WIRELESS MOUSETRAP MONITORING SYSTEM by Alberto Ricci Bitti (c) 2003      www.riccibitti.com
//
//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// MODULE:   MAIN.C
// PURPOSE:  Mousetrap Monitoring System - Main application body and user interface
// REQUIRES: No special requirements.
// NOTES:    Runs in user-monitor mode
//
//////////////////////////////////////////////////////////////////////////////////////////////////////

#include <startup.h>
#include "hardware.h"
#include "keyboard.h"
#include "lcd.h"
#include "radio.h"
#include "traps.h"


#define TIMEOUT_AUTOSCROLL          4                       //each trap is displayed for 4 seconds
#define TIMEOUT_SETUP               20                      //if no user intervention, setup menu exits after 20 seconds
#define TIMEOUT_MANUAL              20                      //if no user intervention, after 20 seconds go back to automatic mode


enum {MODE_AUTO, MODE_MANUAL, MODE_SETUP};                  //menu modes definition

static unsigned char mode           = MODE_AUTO;            //current menu mode
static unsigned char timeout        = 0;                    //timer used to get back to automatic mode or to scroll readings
static unsigned char trap_index     = INDEX_NONE;           //number of the trap memory being browsed
static unsigned int  setup_ID       = ID_NONE;              //last valid ID received, to use in setup menu



//////////////////////////////////////////////////////////////////////////////////////////////////////  
// menu service routine, retrieves trap ID from its memory array index and pretty-prints it to LCD  //
//////////////////////////////////////////////////////////////////////////////////////////////////////          

static void print_trap_ID( unsigned char index )    
{                                                   
    if ( trap_is_free( index ) )
        LCD_puts("  FREE");
    else
    {   LCD_puts( "  ID:" );  LCD_putn( trap_ID[trap_index] ); };
}



//////////////////////////////////////////////////////////////////////////////////////////////////////          
// MENU_SETUP - handles user interface in setup mode                                                //
// The display shows current trap memory number, and wether the memory is FREE or assigned to       //
// a trap ID. Shoud any 'test' signal be received from a mousetrap, it is also shown.               //
// The user can browse all traps memories with FORWARD and BACK keys.                               //
// Pressing CLEAR frees current memory.                                                             //
// Pressing STORE assigns the ID of the received mousetrap to the current memory.                   //
// If there is no user action for some time, system passes automatically to automatic display mode. //
//////////////////////////////////////////////////////////////////////////////////////////////////////          

static void menu_setup( void )
{
    /////////////////////////////////////////////////   
    //       SETUP MODE DISPLAY LAYOUT             //
    /////////////////////////////////////////////////

    LCD_locate(0,0); 
    LCD_puts( "MEMORY#" );  LCD_putn( trap_index+1 );
    print_trap_ID( trap_index );
    
    LCD_clear_to(1,0);
    LCD_puts( "RECEIVED ID:" ); LCD_putn(setup_ID);
    
    LCD_clear_to(0,0);
    

    /////////////////////////////////////////////////   
    //       SETUP MODE    KEYBOARD INTERPRETER    //
    /////////////////////////////////////////////////

    switch ( key_get() )
    {
    case KEY_SETUP:                                         //SETUP key:
        mode = MODE_AUTO;                                   //brings back to automatic mode
        timeout = TIMEOUT_AUTOSCROLL;
        break;
        
    case KEY_FWD:                                           //FORWARD key selects next trap memory 
        trap_index = trap_next( trap_index );           
        timeout = TIMEOUT_SETUP;                            //user pressed a key, reset setup-mode timeout
        break;
        
    case KEY_BACK:                                          //BACK key selects previous trap memory among
        trap_index = trap_prev( trap_index );           
        timeout = TIMEOUT_SETUP;                            //user pressed a key, reset setup-mode timeout
        break;

    case KEY_STORE:                                         //STORE key: 
        if ( (setup_ID != ID_NONE) && is_ID_new(setup_ID) ) //if a valid code has been received AND it is
            trap_store_ID( setup_ID, trap_index );          //not already stored, store it to current memory
        timeout = TIMEOUT_SETUP;                            //user pressed a key, reset setup-modetimeout
        break;

    case KEY_CLEAR:                                         //CLEAR key:
        trap_store_ID( ID_NONE, trap_index );               //put a void ID in current memory
        timeout = TIMEOUT_SETUP;                            //user pressed a key, reset setup-mode timeout
        break;
    }
}



//////////////////////////////////////////////////////////////////////////////////////////////////////          
// MENU_AUTO - handles user interface in automatic and manual mode                                  //
// In automatic mode, the display shows sequentially all trap messages, or the SYSTEM RUNNING       //
// message if no trap requires attention.                                                           //
// Manual mode is like automatic mode, except that the user can browse all traps (including         //
// these that don't ask for attention) and auto-sroll is disabled.                                  //
// Enabled keys are FORWARD, BACK (to browse traps) and SETUP key (to enter SETUP mode)             //
// Pressing CLEAR while a trap is shown, resets its alarm messages                                  //
// If there is no user action for some time, system passes automatically to automatic display mode. //
//////////////////////////////////////////////////////////////////////////////////////////////////////          

void menu_auto(void)
{
    if (trap_index == INDEX_NONE)                           //if no traps selected
        trap_index = next_yelling(trap_index);              //try to get one who needs attention
    
    /////////////////////////////////////////////////   
    //       AUTO & MANUAL DISPLAY LAYOUT          //
    /////////////////////////////////////////////////
    
    LCD_locate(0,0);
    if (trap_index == INDEX_NONE)                           //still no traps to show?
    {
        LCD_puts(" SYSTEM RUNNING   ALL TRAPS OK");         //yes, display normal operation message
    }
    else                                                    //otherwise report trap status
    {
        LCD_puts( "TRAP# "); LCD_putn( trap_index+1 );
        print_trap_ID( trap_index );
        LCD_clear_to(1,0);
        if      (trap_ID[ trap_index ] == ID_NONE)  LCD_puts("-------");
        else if (trap[ trap_index ].timer == 0)     LCD_puts("LOST SIGNAL");
        else if (trap[ trap_index ].trigger)        LCD_puts("TRIGGERED!");
        else if (trap[ trap_index ].battery)        LCD_puts("BATTERY LOW");
        else if (trap[ trap_index ].test)           LCD_puts("TEST SIGNAL");
        else                                        LCD_puts("WORKING OK");
    };  
    LCD_clear_to(0,0);
    
    /////////////////////////////////////////////////   
    //       AUTO & MANUAL KEYBOARD INTERPRETER    //
    /////////////////////////////////////////////////

    switch ( key_get() )
    {
    case KEY_SETUP:                                         //SETUP key pressed:
        if (trap_index == INDEX_NONE)                       //any trap being shown?
            trap_index = 0;                                 //no, default to first trap memory
        mode = MODE_SETUP;                                  //switch to setup mode
        timeout = TIMEOUT_SETUP;                            //reset timeout
        break;

    case KEY_FWD:                                           //FORWARD key pressed:
        trap_index = trap_next( trap_index );               //select next trap memory for display
        mode = MODE_MANUAL;                                 //user selected manually a memory
        timeout = TIMEOUT_MANUAL;                           //user intervention, reset timeout
        break;
        
    case KEY_BACK:                                          //BACK key pressed:
        trap_index = trap_prev( trap_index );               //select previous trap memory for display
        mode = MODE_MANUAL;                                 //user selected manually a memory
        timeout = TIMEOUT_MANUAL;                           //user intervention, reset timeout
        break;

    case KEY_CLEAR:                                         //CLEAR key pressed
        if (trap_index != INDEX_NONE)                       //any trap being shown?
            trap_clear( trap_index );                       //yes, reset trap flags (low battery, heartbeat, trigger)
        timeout = TIMEOUT_AUTOSCROLL;                       //user intervention, reset timeout
        break;
    }
}



//////////////////////////////////////////////////////////////////////////////////////////////////////          
// MAIN - main program endless loop                                                                 //
//  - Performs hardware initializations                                                             //
//  - Receives signals from traps                                                                   //
//  - Updates trap database according to signals received                                           //
//  - Manages user interfaces according to current operating mode and timeouts                      //
//  - Executes periodic tasks (update of keep-alive timers and menu timeouts)                       //
//  - Drives relay otuput                                                                           //
//////////////////////////////////////////////////////////////////////////////////////////////////////          


void main(void) 
{
    T_RADIO_DATA  radio_data = 0;                           //current dataword as received from radio module
    
    hardware_initialize();                                  //initialize all modules
    key_initialize();                                       //when possible, each module initialize
    radio_initialize();                                     //its own I/O pins
    LCD_initialize();                                       //to enhance code portability
    relay_initialize();                                     
    trap_initialize();
    

    for(;;)                                                 //main application loop
    {
        radio_data = radio_get_packet();                    //get last received signal

        trap_update( radio_data );                          //update trap database based on last
                                                            //received signal

        if ( radio_is_test( radio_data ) )                  //did you receive a test signal?
            setup_ID = radio_to_ID( radio_data );           //yes, keep it for setup

    
        switch( mode )                                      //select menu according to current
        {                                                   //operating mode
        case MODE_AUTO:                                     
            menu_auto();                                    //auto mode provides timed scroll
            if (timeout == 0)                               //of trap data:
            {                                               //only traps with relevant information
                trap_index = next_yelling( trap_index );    //("yelling traps") are shown,
                timeout = TIMEOUT_AUTOSCROLL;               //quiet traps are ignored
            };                                              
            break;
            
        case MODE_MANUAL:                                   //when the user selects a trap touching the
            menu_auto();                                    //keyboard, the system switches to manual
            if (timeout == 0)                               //mode. Manual is the same as Auto, except
            {                                               //auto-scroll is disabled.
                mode = MODE_AUTO;                           //If some time passes without any user
                timeout = TIMEOUT_MANUAL;                   //interaction, the system switches back
            };                                              //to automatic mode
            break;
            
        case MODE_SETUP:                        
            menu_setup(); 
            if (timeout == 0)
            {                                               //if some time passes without any user
                mode = MODE_AUTO;                           //interaction, the system switches back
                timeout = TIMEOUT_SETUP;                    //to automatic mode
            };                                              
            break;
        };
                                                            ////////////////////////////////////////
                                                            //housekeeping scheduler follows
                                                            ////////////////////////////////////////

        if ( timeflags.second && timeout != 0 )             //every second, decrement menu timeouts
        {
            timeout--;  
            timeflags.second = FALSE;   
        };
                
                
        if ( timeflags.hour )                               //every hour, update trap heartbeat
        {                                                   //timers. Should a timer be completely
            heartbeat();                                    //discharged, trap signal is deemed lost
            timeflags.minute = FALSE;
        };
        
        RELAY_OUTPUT = (next_yelling(0) != INDEX_NONE);     //if any trap needs attention
                                                            //switch-on the relay
    }
}    


      
