//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// WIRELESS MOUSETRAP MONITORING SYSTEM by Alberto Ricci Bitti (c) 2003      www.riccibitti.com
//
//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// MODULE:   RADIO.C
// PURPOSE:  monitors incoming bit stream from the radio receiver, decodes trap transmissions
//           provides functions to extract single flags from raw codewords (e.g radio_is_trigger())
// REQUIRES: Call radio_poll at a 10 kHz frequency
// NOTES:    None
//
////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "hardware.h"
#include "radio.h"


#define NUM_BITS 11


// RF packet bit fields:
//---------------------------------------------
// START    1 bit       must be always 1
// ID       7 bits      identifies trap
// TRIGGER  1 bit       trap is triggered
// BATTERY  1 bit       trap low battery flag
// TEST     1 bit       trap is in test/reduced power mode
//
//---------------------------------------------
//
// RF waveform encoding:
//
//
//
//        logic 1 = .66 mS .33 mS
//                  +-----+--+
//
//
//                logic 0 = .33 mS .66 mS
//                           +--+-----+
//
//               +--+     +--+  +--+--+  +--+     +--
//               |  |     |  |  |     |  |  |     |
//----....-------+  +--+--+  +--+     +--+  +--+--+
//
//
//  sync pause   start    1' bit   2'bit    3'bit     ... total: 10 bits + start
//(min 11.52 mS) 


//min and max pulse duration, according to the above waveform sampled @ 10 kHz
//
#define MAX_ONE 4
#define MIN_ONE 2
#define MAX_ZERO 8
#define MIN_ZERO 5
#define MIN_PAUSE MIN_ONE
#define MAX_PAUSE MAX_ZERO
#define MIN_SYNC 110

static T_RADIO_DATA  received = 0;      //buffer for last valid data recived



///////////////////////////////////////////////////////////////////////////////
// radio_initialize - initialize receiver I/O                                //
///////////////////////////////////////////////////////////////////////////////

void radio_initialize(void)
{
    RADIO_PULLUP = 1;                   //turn on pullups
    RADIO_DDR    = 0;                   //and set port direction to input
}



///////////////////////////////////////////////////////////////////////////////
// radio_poll - samples the receiver input and decodes incoming data         //
///////////////////////////////////////////////////////////////////////////////

void radio_poll( void )
{
    enum { WAIT_SYNC, GET_BIT, GET_PAUSE };                         //radio decoder states 
    static unsigned char mode  = WAIT_SYNC;                         //current decoder state
    static unsigned char width = 0;                                 //measures pulse width
    static unsigned char bits = 0;                                  //counts bit received
    static T_RADIO_DATA  packet = 0;                                //working buffer for code being received
    static T_RADIO_DATA  pvs_packet = 0;                            //codes must be received twice before being accepted

    switch ( mode )
    {
    case WAIT_SYNC:
        if (RADIO_INPUT == 0)                                       //is pause end?
            width++;                                                //no, keep measuring pause width
        else if (width > MIN_SYNC)                                  //pause ended: was it long enough
        {                                                           //to qualify for a SYNC pause?
            bits = 0;   
            packet = 0;                                             //yes, clear buffers and prepare  
            mode = GET_BIT;                                         //to receive first bit of sequence
            width = 0;                                              
        }
        else
            width = 0;                                              //otherwise keep loking for
        break;                                                      //a SYNC pause
        
    case GET_BIT:
        if (RADIO_INPUT == 1)                                       //is pulse end?
            width++;                                                //no, keep measuring pulse width
        else
        {                                                           //otherwise check if it was a well-shaped pulse
            if ( (width >= MIN_ONE) && (width <= MAX_ONE) )         //is pulse width within limits for "1"?
            {   
                packet = (packet << 1) | 1;                         //yes, got a valid "1"
                bits++;                                             //shift it in, count it,
                mode = GET_PAUSE;                                   //and prepare for a pause
            }
            else if ( (width >= MIN_ZERO) && (width <= MAX_ZERO) )  //is pulse width within limits for "0"?
            {   
                packet = packet << 1;                               //yes, got a valid "0"
                bits++;                                             //shift it in, count it
                mode = GET_PAUSE;                                   //and prepare for a pause
            }
            else
            {                                                        
                mode = WAIT_SYNC;                                   //width is neither a valid one or zero
            };                                                      //therefore abort reception
            

            if (bits == NUM_BITS)                                   //got all bits?
            {
                    if (pvs_packet == packet)                       //got the same code twice?
                        received = packet;                          //yes, copy complete code to result buffer
                        
                    pvs_packet = packet;                            //store current reception for next redundancy check
                    mode = WAIT_SYNC;                               //and start a new reception
            } else if (bits == 1)                                   //first bit?
            {

                if ( (packet & 1) != 1 )                            //yes, then it must be "1"
                {
                    mode = WAIT_SYNC;                               //otherwise abort reception 
                };                                                  //and wait for a new sync
            };
            width = 0;
        };
        break;

    case GET_PAUSE:
        if (RADIO_INPUT == 0)                                       //is pause end?
            width++;                                                //no, keep meausring pause width
        else
        {                                                           //otherwise check if it was of correct duration
            if ( (width >= MIN_PAUSE) && (width <= MAX_PAUSE) )     //is width within pause limits?
            {   
                mode = GET_BIT;                                     //yes, got a valid pause
                width = 0;                                          //prepare to receive a new bit
            }
            else                                                    
            {                                                       
                mode = WAIT_SYNC;                                   //otherwise reject abort reception
                width = 0;                                          //and wait for a new sync
            };
        };
        break;
        
    }; /*switch(mode)*/
    
    if (width == 255)                                               //prevent "width" from overflows 
            width--;                                                //that could result in false readings
}



///////////////////////////////////////////////////////////////////////////////
// radio_get_packet- returns the last data received and clears               //
// the receive buffer                                                        //
///////////////////////////////////////////////////////////////////////////////

T_RADIO_DATA radio_get_packet( void )
{
    unsigned int retval;
    
    DisableInterrupts;                          //"received" cannot be passed diretcly to main program,
    retval = received;                          //because it is modified by an ISR
    received = 0;                               //to touch it, disable interrupt and use a buffer
    EnableInterrupts;                               
    return retval;
}



///////////////////////////////////////////////////////////////////////////////
// radio_to_ID - return the ID from raw data                                 //
///////////////////////////////////////////////////////////////////////////////

unsigned int radio_to_ID( unsigned int raw )
{
    return (raw >> 3) & 0x7F;
}



///////////////////////////////////////////////////////////////////////////////
// radio_is_trigger - returns TRUE if the codeword comes from a              //
// triggered trap                                                            //
///////////////////////////////////////////////////////////////////////////////
 
unsigned char radio_is_trigger( unsigned int raw )
{
    return  ((raw & ( 1 << 2)) != 0 );
}



///////////////////////////////////////////////////////////////////////////////
// radio_is_battery - returns TRUE if the codeword comes from a              //
// trap with battery partially discharged                                    //                                    
///////////////////////////////////////////////////////////////////////////////

unsigned char radio_is_battery( unsigned int raw )  
{
    return ( (raw & ( 1 << 1)) != 0 );
}



///////////////////////////////////////////////////////////////////////////////
// radio_is_trigger - returns TRUE if the codeword comes from a trap where   //
// the user just pressed the REARM button                                    //
///////////////////////////////////////////////////////////////////////////////

unsigned char radio_is_test( unsigned int raw )
{ 
    return ((raw & ( 1 << 0)) != 0 );
}






    
    


