//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// WIRELESS MOUSETRAP MONITORING SYSTEM by Alberto Ricci Bitti (c) 2003      www.riccibitti.com
//
//////////////////////////////////////////////////////////////////////////////////////////////////////  


#include "hardware.h"
#include "flash.h"
#include "traps.h"


///////////////////////////////////////////////////////////////////////////////
// our tiny trap database, each trap has its own descriptor                  //
///////////////////////////////////////////////////////////////////////////////

T_TRAP_DESCRIPTOR trap[ NUM_MEMORIES ] = {
    0,0,0,0, 
    0,0,0,0, 
    0,0,0,0, 
    0,0,0,0, 
    0,0,0,0
};  



///////////////////////////////////////////////////////////////////////////////
//trap module IDs are stored in flash memory using a special segment named   //
//FLASH_EEPROM for eeprom emulation                                          //
///////////////////////////////////////////////////////////////////////////////

#pragma CONST_SEG FLASH_EEPROM 

const unsigned int trap_ID[ NUM_MEMORIES ] = {
    ID_NONE, ID_NONE, ID_NONE, ID_NONE,
    ID_NONE, ID_NONE, ID_NONE, ID_NONE,
    ID_NONE, ID_NONE, ID_NONE, ID_NONE,
    ID_NONE, ID_NONE, ID_NONE, ID_NONE,
    ID_NONE, ID_NONE, ID_NONE, ID_NONE
} ;

#pragma CONST_SEG DEFAULT       //back to deafult memory segment



///////////////////////////////////////////////////////////////////////////////
// trap_clear - resets trap record                                           //
///////////////////////////////////////////////////////////////////////////////

void trap_clear( unsigned char index )
{
    trap[ index ].trigger = FALSE;
    trap[ index ].battery = FALSE;
    trap[ index ].test    = FALSE;
    trap[ index ].timer   = TRAP_HEARTBEAT_TIME;
}



///////////////////////////////////////////////////////////////////////////////
// trap_initailize - initialize trap database                                //
///////////////////////////////////////////////////////////////////////////////

void trap_initialize(void)
{
    unsigned char n;

    for (n=0; n< NUM_MEMORIES; n++)
        trap_clear( n );
}


///////////////////////////////////////////////////////////////////////////////
//trap_update - updates trap descriptors database according to the           //
//radio codeword received                                                    //
///////////////////////////////////////////////////////////////////////////////

void trap_update( T_RADIO_DATA radio )
{
    unsigned char index = ID_to_index( radio_to_ID( radio ) ); //index or the trap as received
    
    if (index != INDEX_NONE )                                  //does current data identify a stored trap ID?
    {                                                          //yes, update trap database
        trap[ index ].trigger |= radio_is_trigger( radio );    //set trap flags...
        trap[ index ].battery |= radio_is_battery( radio );    //
        trap[ index ].test    |= radio_is_test   ( radio );    //
        trap[ index ].timer    = TRAP_HEARTBEAT_TIME;          //...and reset timer to reflect time from last reception
    };
}           



///////////////////////////////////////////////////////////////////////////////
// heartbeat - decrement heartbeat event counter for all traps               //
//  -  a trap is deemed lost when its heartbeat reaches zero                 // 
//  - the heartbead is reloaded when trap data is received                   //
///////////////////////////////////////////////////////////////////////////////

void heartbeat()
{   
    unsigned char n;

    for (n=0; n< NUM_MEMORIES; n++)
    if (trap[n].timer > 0)
                trap[n].timer--;
}



///////////////////////////////////////////////////////////////////////////////
// next_yelling - starting from the specified index, return the next trap    //
// which  must be signalled to the user ("yellling").                        //
// A yelling trap must have a valid ID (free memory indexes are disregarded) //
// and must satisfy at least one of the following conditions:                //
//  - the user just pressed REARM (test mode)                                //
//  - the trap has a discharged battery                                      //
//  - the trap is triggered                                                  //
//  - the trap is lost                                                       //
///////////////////////////////////////////////////////////////////////////////

unsigned char next_yelling( unsigned char index )
{
    unsigned char n;
    T_TRAP_DESCRIPTOR trp; 
    
    for ( n=0; n<NUM_MEMORIES; n++ )
    {
        if ( ++index >= NUM_MEMORIES )
            index = 0;
        trp = trap[ index ];
        
        if ( trap_ID[index] != ID_NONE )
        {
            if ( trp.test || trp.battery || trp.trigger  || (trp.timer == 0) )
                return index;
        }
    };
    return INDEX_NONE;
}



///////////////////////////////////////////////////////////////////////////////
// ID_to_index - returns the memory index of the trap in FLASH array that    //
// matches the ID, or returns INDEX_NONE if no matching ID found             //
///////////////////////////////////////////////////////////////////////////////

unsigned char ID_to_index( unsigned int ID )
{
    unsigned char n = INDEX_NONE;
    
    if ( ID != ID_NONE )
    {
        for ( n=0; n<NUM_MEMORIES; n++ )
        {
            if ( trap_ID[n] == ID )
            {
                break;
            };
        };
    };
    return n;
}



///////////////////////////////////////////////////////////////////////////////
// is_ID_new - returns TRUE if the given trap ID does not match any of the   //
// known IDs                                                                 //
///////////////////////////////////////////////////////////////////////////////

unsigned char is_ID_new( unsigned int ID )
{
    return ID_to_index( ID ) == INDEX_NONE;
}



///////////////////////////////////////////////////////////////////////////////
// trap_is_free - returns TRUE if the trap memory is unassigned to any ID    //
///////////////////////////////////////////////////////////////////////////////

unsigned char trap_is_free( unsigned char index )
{
    return trap_ID[index] == ID_NONE;
}



///////////////////////////////////////////////////////////////////////////////
// trap_next - returns the index of the following trap memory, with rollover //
///////////////////////////////////////////////////////////////////////////////

unsigned char trap_next( unsigned char index )
{
    if ( index >= NUM_MEMORIES-1 )
        return 0;
    else
        return index + 1;
}



///////////////////////////////////////////////////////////////////////////////
// trap_prev - returns the index of the preceding trap memory, with rollover //
///////////////////////////////////////////////////////////////////////////////

unsigned char trap_prev( unsigned char index )
{
    if (index == 0)
        return NUM_MEMORIES-1;
    else
        return index - 1;
}



///////////////////////////////////////////////////////////////////////////////
// trap_store_ID - stores ID in flash memory, using EEPROM emulation         //
///////////////////////////////////////////////////////////////////////////////

void trap_store_ID(unsigned int ID, unsigned char index)
{   
    unsigned char i;
    const unsigned int * start_addr;
    const unsigned int * dest_addr;
    
    flash_erase_page( (const unsigned char *) FLASH_UPPER_PAGE_ADDRESS );   //first, erase flash copy buffer
                                                                            //in upper flash page


    start_addr =  &trap_ID[0];                                              //second, copy there all source data
    dest_addr  =  FLASH_UPPER_PAGE_ADDRESS;
    for (i=0; i<NUM_MEMORIES; i++, start_addr++, dest_addr++)
    {
            flash_program_word( dest_addr, *start_addr);
    };
    
    flash_erase_page( (const unsigned char *) FLASH_LOWER_PAGE_ADDRESS );   //now the upper page buffer holds a copy 
                                                                            // of the ID table, so we can erase the 
                                                                            //lower page

    start_addr = FLASH_UPPER_PAGE_ADDRESS;                                  //copy data back in the original place,
    dest_addr  = & trap_ID[0];                                              //but replace the new entry
    for (i=0; i<NUM_MEMORIES; i++, start_addr++, dest_addr++)
    {
        if (i != index)
            flash_program_word( dest_addr, *start_addr);
        else
            flash_program_word( dest_addr, ID);
    }; 
}
