//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// WIRELESS MOUSETRAP MONITORING SYSTEM by Alberto Ricci Bitti (c) 2003      www.riccibitti.com
//
//////////////////////////////////////////////////////////////////////////////////////////////////////  
//
// MODULE:   MAIN.C
// PURPOSE:  Mousetrap Monitoring System - Mousetrap transmitter main application
// REQUIRES: No special requirements.
// NOTES:    Runs in user-monitor mode
//
//////////////////////////////////////////////////////////////////////////////////////////////////////


#include <hidef.h>
#include <startup.h>
#include "hardware.h"
#include "flash.h"


//              OUTPUT WAVEFORM TO RF TRANSMITTER
//
//        logic 1 = .66 mS .33 mS
//                  +-----+--+
//
//
//                logic 0 = .33 mS .66 mS
//                           +--+-----+
//
//
//               +--+     +--+  +--+--+  +--+     +--
//               |  |     |  |  |     |  |  |     |
//----....-------+  +--+--+  +--+     +--+  +--+--+
//
//
//  sync pause   start    1' bit   2'bit    3'bit     ... total: 12 bits + start
//(11 mS minimum) 

#define CODE_BITS 			11
#define START_BIT 			( 1 << (CODE_BITS-1) )
#define BIT_THIRD_DURATION	0.33E-3						//bit time is split in three 0.33 mS slices
#define PAUSE_TIME_mS    	12 							//minimum pause duration between TX repetitions

#define NUM_ID				128							//maximum number of different trap IDs

#define HEARTBEAT_SECONDS 	(60 * 60)					//60 sec * 60 min = transmit every hour

#define ID_FLASH_ADDR		((unsigned char *) 0xEE00)	//address in FLASH of the ID (see flash.h)


//////////////////////////////////////////////////////////////////////////////////////////////////////
// rf_encoder - encodes and outputs the codeword according to the RF transmission format
//
// NOTE: puts the processor in WAIT mode to reduce power
/////////////////////////////////////////////////////////////////////////////////////////////////////

static void rf_encoder(unsigned int codeword)
{
	unsigned int mask;
	
	TMOD = CLOCK_FREQ * BIT_THIRD_DURATION; 			//prepare for timer overflow every .33 ms (a bit third)
														//overflow will wakeup from WAIT mode
	
	mask = 1 << (CODE_BITS - 1);		    			//prepare mask for filtering leftmost bit

	while( mask != 0 )
	{
		TSC_TSTOP = 0;					    			//restart timer
		WAIT_MODE;						    			//go in low power mode until a .33 mS period expires
		if ( (codeword & mask) == 0 )	    			//are we transmitting a zero?
					TX_ON;				    			//yes, force a 66% duty cycle
		WAIT_MODE;						    			//go in low power mode until a .33 mS period expires
		TX_ON;							    			//set output to ensure a duty cicle be not less than 33%
		WAIT_MODE;						    			//go in low power mode until a .33 mS period expires
		TX_OFF;							    			//turn transmitter off
		mask >>= 1;						    			//prepare mask for filtering next bit
	};
	TSC_TSTOP = 1;						    			//stop timer before exit to reduce power consumption
}



//////////////////////////////////////////////////////////////////////////////////////////////////////
// wait_mS - waits the specified number of milliseconds
//
// NOTE: puts the processor in WAIT mode to reduce power
//////////////////////////////////////////////////////////////////////////////////////////////////////

static void wait_mS( int mS )
{
	TMOD      = CLOCK_FREQ * 1E-3;						//prepare for timer overflow every mS 
	TSC_TRST  = 1;										//reset timer count to 0
	TSC_TSTOP = 0;										//and start counting
	while (mS-- > 0)									//repeat loop for all of the milliseconds required
		WAIT_MODE;										//go in low-power wait mode until timer overflows
	TSC_TSTOP = 1;										//stop timer again to reduce power consumption
	
}



//////////////////////////////////////////////////////////////////////////////////////////////////////
// transmit - assembles a codeword according to the status flags and ID, and transmits it mant times
//////////////////////////////////////////////////////////////////////////////////////////////////////

static void transmit( unsigned char repetitions, unsigned char ID, unsigned char flag_trigger, unsigned char flag_low_bat, unsigned char flag_test)
{
	unsigned int codeword;
	
	//assemble bits into a complete codeword
	codeword = START_BIT | (ID & 0x7F) <<3 | flag_trigger << 2 | flag_low_bat << 1 | flag_test;

	while (repetitions--)
	{
		rf_encoder( codeword );
		wait_mS( PAUSE_TIME_mS + ID % 16 );				//repeat rate varies with ID in order to
	};													//prevent data collisions
}



//////////////////////////////////////////////////////////////////////////////////////////////////////
// get_intertal - returns an ID-dependent interval (in wakeup timer ticks) to be used
// to repeat heartbeat transmissions
//////////////////////////////////////////////////////////////////////////////////////////////////////

static unsigned int get_interval( unsigned char ID )
{
	return ID + HEARTBEAT_SECONDS * 2;					//the wakeup timer is about half a second, 
}														//multiply seconds by 2 to get interval count



//////////////////////////////////////////////////////////////////////////////////////////////////////
// main - main transmitter loop
// - initializes hardware register, interrupts etc.
// - checks if any of the inputs triggers, updates status flags and informs the monitoring station
// - transmits a cyclic heartbeat to signal its presence to the monitoring station
// - modifies/store trap ID in FLASH using EEPROM emulation
//////////////////////////////////////////////////////////////////////////////////////////////////////

void main(void)
{ 
	unsigned char trigger  	= FALSE;					//flag, trap triggered
	unsigned char low_bat  	= FALSE;					//flag, time to change battery
	unsigned int  interval 	= 0;						//time (in wake-up timer ticks) between
														//"heartbeat" transmissions
	unsigned char ID 		= *ID_FLASH_ADDR;			//copy non-volatile value from FLASH

	hardware_initialize();								//initialize ports, registers and interrupts
    EnableInterrupts;									//make interrupts active

	
	for(;;)												//endless main loop
   	{
	
	
		stop_mode();									//go in low power mode
														//awaken only by a time quantum, keypress 
														//or mouse sensor triggering

		if ( LVISR_LVIOUT ) 							//is battery going flat?
		{
			low_bat = TRUE;								//yes, inform control station
		};

		if (REED_INPUT == 0)							//is the mouse playing with
	    {												//balance?
	    	trigger = TRUE;								//yes, flag trap triggered status
	      	REED_DISABLE;								//and disable input to save power
	    };

	    if ( REARM_INPUT == 0 )							//is REARM key pressed?
	    {
	    	trigger = FALSE;							//yes, reset trap trigger and 
	    	low_bat = FALSE;							//reset low battery flags and
	    	REED_ENABLE;								//re-enable mouse sensor input and
            transmit( 20, ID, trigger, low_bat, TRUE );	//let control station knows new status
	    };

	    if ( SETUP_INPUT == 0)							//is SETUP key pressed?
	    {
	    	ID = (ID+1) % NUM_ID;						//yes, generate new ID
	    	flash_erase_page(ID_FLASH_ADDR);			//and store it to 
	    	flash_program_byte(ID_FLASH_ADDR, ID);		//non-volatile (flash) memory		
            transmit( 20, ID, trigger, low_bat, TRUE);	//let control station learn new ID
	    };

		if (interval-- == 0)							//decrement transmission interval counter
		{												//is it time to transmit?
            transmit( 5, ID, trigger, low_bat, FALSE );	//yes, transmit more recent flags
            interval = get_interval( ID );				//and reload interval counter according to ID
	    };
   };
}    
       
